"use strict";
/**
 * @license
 * Copyright Google Inc. All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateHelpMessage = exports.parseCommandLine = void 0;
function findArgument(argv, argName) {
    const index = argv.indexOf(argName);
    if (index < 0 || index === argv.length - 1) {
        return;
    }
    return argv[index + 1];
}
function parseStringArray(argv, argName) {
    const arg = findArgument(argv, argName);
    if (!arg) {
        return [];
    }
    return arg.split(',');
}
function hasArgument(argv, argName) {
    return argv.includes(argName);
}
function parseCommandLine(argv) {
    return {
        help: hasArgument(argv, '--help'),
        ivy: !hasArgument(argv, '--viewEngine'),
        disableAutomaticNgcc: hasArgument(argv, '--disableAutomaticNgcc'),
        logFile: findArgument(argv, '--logFile'),
        logVerbosity: findArgument(argv, '--logVerbosity'),
        logToConsole: hasArgument(argv, '--logToConsole'),
        ngProbeLocations: parseStringArray(argv, '--ngProbeLocations'),
        tsProbeLocations: parseStringArray(argv, '--tsProbeLocations'),
        includeAutomaticOptionalChainCompletions: hasArgument(argv, '--includeAutomaticOptionalChainCompletions'),
        includeCompletionsWithSnippetText: hasArgument(argv, '--includeCompletionsWithSnippetText'),
        forceStrictTemplates: hasArgument(argv, '--forceStrictTemplates'),
    };
}
exports.parseCommandLine = parseCommandLine;
function generateHelpMessage(argv) {
    return `Angular Language Service that implements the Language Server Protocol (LSP).

  Usage: ${argv[0]} ${argv[1]} [options]

  Options:
    --help: Prints help message.
    --viewEngine: Use legacy View Engine language service. Defaults to false.
    --logFile: Location to log messages. Logging to file is disabled if not provided.
    --logVerbosity: terse|normal|verbose|requestTime. See ts.server.LogLevel.
    --logToConsole: Enables logging to console via 'window/logMessage'. Defaults to false.
    --ngProbeLocations: Path of pugjs-angular-language-service. Required.
    --tsProbeLocations: Path of typescript. Required.

  Additional options supported by vscode-languageserver:
    --clientProcessId=<number>: Automatically kills the server if the client process dies.
    --node-ipc: Communicate using Node's IPC. This is the default.
    --stdio: Communicate over stdin/stdout.
    --socket=<number>: Communicate using Unix socket.
  `;
}
exports.generateHelpMessage = generateHelpMessage;
//# sourceMappingURL=cmdline_utils.js.map