"use strict";
/**
 * @license
 * Copyright Google Inc. All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPugCompletions = exports.getHtmlCompletions = exports.tsCompletionEntryToLspCompletionItem = exports.readNgCompletionData = void 0;
const ts = __importStar(require("typescript/lib/tsserverlibrary"));
const lsp = __importStar(require("vscode-languageserver"));
const utils = __importStar(require("./utils/utils"));
const vscode_html_languageservice_1 = require("vscode-html-languageservice");
const SourceMap = require("@volar/source-map");
// TODO: Move this to `pugjs-angular-language-service`.
var CompletionKind;
(function (CompletionKind) {
    CompletionKind["attribute"] = "attribute";
    CompletionKind["htmlAttribute"] = "html attribute";
    CompletionKind["property"] = "property";
    CompletionKind["component"] = "component";
    CompletionKind["directive"] = "directive";
    CompletionKind["element"] = "element";
    CompletionKind["event"] = "event";
    CompletionKind["key"] = "key";
    CompletionKind["method"] = "method";
    CompletionKind["pipe"] = "pipe";
    CompletionKind["type"] = "type";
    CompletionKind["reference"] = "reference";
    CompletionKind["variable"] = "variable";
    CompletionKind["entity"] = "entity";
})(CompletionKind || (CompletionKind = {}));
/**
 * Extract `NgCompletionOriginData` from an `lsp.CompletionItem` if present.
 */
function readNgCompletionData(item) {
    if (item.data === undefined) {
        return null;
    }
    // Validate that `item.data.kind` is actually the right tag, and narrow its type in the process.
    const data = item.data;
    if (data.kind !== 'ngCompletionOriginData') {
        return null;
    }
    return data;
}
exports.readNgCompletionData = readNgCompletionData;
/**
 * Convert Angular's CompletionKind to LSP CompletionItemKind.
 * @param kind Angular's CompletionKind
 */
function ngCompletionKindToLspCompletionItemKind(kind) {
    switch (kind) {
        case CompletionKind.attribute:
        case CompletionKind.htmlAttribute:
        case CompletionKind.property:
        case CompletionKind.event:
            return lsp.CompletionItemKind.Property;
        case CompletionKind.directive:
        case CompletionKind.component:
        case CompletionKind.element:
        case CompletionKind.key:
            return lsp.CompletionItemKind.Class;
        case CompletionKind.method:
            return lsp.CompletionItemKind.Method;
        case CompletionKind.pipe:
            return lsp.CompletionItemKind.Function;
        case CompletionKind.type:
            return lsp.CompletionItemKind.Interface;
        case CompletionKind.reference:
        case CompletionKind.variable:
            return lsp.CompletionItemKind.Variable;
        case CompletionKind.entity:
        default:
            return lsp.CompletionItemKind.Text;
    }
}
/**
 * Convert ts.CompletionEntry to LSP Completion Item.
 * @param entry completion entry
 * @param position position where completion is requested.
 * @param scriptInfo
 */
function tsCompletionEntryToLspCompletionItem(entry, position, scriptInfo, insertReplaceSupport, isIvy, document) {
    const item = lsp.CompletionItem.create(entry.name);
    // Even though `entry.kind` is typed as ts.ScriptElementKind, it's
    // really Angular's CompletionKind. This is because ts.ScriptElementKind does
    // not sufficiently capture the HTML entities.
    // This is a limitation of being a tsserver plugin.
    const kind = entry.kind;
    item.kind = ngCompletionKindToLspCompletionItemKind(kind);
    item.detail = entry.kind;
    item.sortText = entry.sortText;
    // Text that actually gets inserted to the document. It could be different
    // from 'entry.name'. For example, a method name could be 'greet', but the
    // insertText is 'greet()'.
    const insertText = entry.insertText || entry.name;
    if (document) {
        item.textEdit = createTextEdit(document, entry, position, insertText, insertReplaceSupport);
    }
    else {
        item.textEdit = createTextEdit(scriptInfo, entry, position, insertText, insertReplaceSupport);
    }
    if (isIvy) {
        // If the user enables the config `includeAutomaticOptionalChainCompletions`, the `insertText`
        // range will include the dot. the `insertText` should be assigned to the `filterText` to filter
        // the completion items.
        item.filterText = entry.insertText;
        if (entry.isSnippet) {
            item.insertTextFormat = lsp.InsertTextFormat.Snippet;
        }
    }
    item.data = {
        kind: 'ngCompletionOriginData',
        filePath: scriptInfo.fileName,
        position,
    };
    return item;
}
exports.tsCompletionEntryToLspCompletionItem = tsCompletionEntryToLspCompletionItem;
function createTextEdit(file, entry, position, insertText, insertReplaceSupport) {
    if (file instanceof ts.server.ScriptInfo) {
        return _createTextEditScriptInfo(file, entry, position, insertText, insertReplaceSupport);
    }
    else {
        return _createTextEditTextDocument(file, entry, position, insertText, insertReplaceSupport);
    }
}
function getHtmlCompletions(scriptInfo, ngLs, position, options, clientSupportsInsertReplaceCompletion, ivy) {
    const offset = utils.lspPositionToTsPosition(scriptInfo, position);
    const ngCompletions = ngLs.getCompletionsAtPosition(scriptInfo.fileName, offset, options);
    if (ngCompletions === null || ngCompletions === void 0 ? void 0 : ngCompletions.entries.length) {
        return ngCompletions.entries.map((entry) => {
            return tsCompletionEntryToLspCompletionItem(entry, position, scriptInfo, clientSupportsInsertReplaceCompletion, ivy);
        });
    }
    return null;
}
exports.getHtmlCompletions = getHtmlCompletions;
function _createTextEditTextDocument(document, entry, position, insertText, insertReplaceSupport) {
    if (entry.replacementSpan === undefined) {
        return lsp.TextEdit.insert(position, insertText);
    }
    else if (insertReplaceSupport) {
        const replacementRange = utils.tsTextSpanToLspRange(document, entry.replacementSpan);
        const tsPosition = utils.lspPositionToTsPosition(document, position);
        const insertLength = document.offsetAt(position) - entry.replacementSpan.start;
        const insertionRange = utils.tsTextSpanToLspRange(document, Object.assign(Object.assign({}, entry.replacementSpan), { length: insertLength }));
        return lsp.InsertReplaceEdit.create(insertText, insertionRange, replacementRange);
    }
    else {
        return lsp.TextEdit.replace(utils.tsTextSpanToLspRange(document, entry.replacementSpan), insertText);
    }
}
function _createTextEditScriptInfo(scriptInfo, entry, position, insertText, insertReplaceSupport) {
    if (entry.replacementSpan === undefined) {
        return lsp.TextEdit.insert(position, insertText);
    }
    else if (insertReplaceSupport) {
        const replacementRange = utils.tsTextSpanToLspRange(scriptInfo, entry.replacementSpan);
        const tsPosition = utils.lspPositionToTsPosition(scriptInfo, position);
        const insertLength = tsPosition - entry.replacementSpan.start;
        const insertionRange = utils.tsTextSpanToLspRange(scriptInfo, Object.assign(Object.assign({}, entry.replacementSpan), { length: insertLength }));
        return lsp.InsertReplaceEdit.create(insertText, insertionRange, replacementRange);
    }
    else {
        return lsp.TextEdit.replace(utils.tsTextSpanToLspRange(scriptInfo, entry.replacementSpan), insertText);
    }
}
async function getPugCompletions(pugDocument, pugCursorPosition, htmlCursorOffset, pugCursorOffset, pugLs) {
    var _a, _b;
    const plusOnePugPosition = pugDocument.pugTextDocument.positionAt(pugDocument.pugTextDocument.offsetAt(pugCursorPosition) + 1);
    let newPugDoc = pugDocument;
    if (((_b = (_a = pugDocument.sourceMap.getMappedRange(pugCursorPosition)) === null || _a === void 0 ? void 0 : _a[1]) === null || _b === void 0 ? void 0 : _b.text) === 'newline') {
        pugDocument.sourceMap.mappings;
        const newText = pugDocument.htmlTextDocument.getText().substring(0, htmlCursorOffset - 0) + '<' + pugDocument.htmlTextDocument.getText().substring(htmlCursorOffset + 1 - 1);
        const newDocument = vscode_html_languageservice_1.TextDocument.create('foo.html', 'html', 0, newText);
        newPugDoc.htmlTextDocument = newDocument;
        newPugDoc.sourceMap.mappings.unshift({
            sourceRange: { start: pugCursorOffset, end: pugCursorOffset },
            mappedRange: { start: htmlCursorOffset + 1, end: htmlCursorOffset + 1 },
            mode: SourceMap.Mode.Totally,
            data: { isEmptyTagCompletion: true, text: 'htmlcorrection' }
        });
    }
    return await pugLs.doComplete(newPugDoc, pugCursorPosition, { resolveReference: (ref, base) => `${ref}/${base}` });
}
exports.getPugCompletions = getPugCompletions;
//# sourceMappingURL=completion.js.map