"use strict";
/**
 * @license
 * Copyright Google Inc. All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tsDiagnosticToLspDiagnostic = exports.pugDiagnosticToLspDiagnostic = void 0;
const ts = __importStar(require("typescript/lib/tsserverlibrary"));
const lsp = __importStar(require("vscode-languageserver"));
const utils = __importStar(require("./utils/utils"));
/**
 * Convert ts.DiagnosticCategory to lsp.DiagnosticSeverity
 * @param category diagnostic category
 */
function tsDiagnosticCategoryToLspDiagnosticSeverity(category) {
    switch (category) {
        case ts.DiagnosticCategory.Warning:
            return lsp.DiagnosticSeverity.Warning;
        case ts.DiagnosticCategory.Error:
            return lsp.DiagnosticSeverity.Error;
        case ts.DiagnosticCategory.Suggestion:
            return lsp.DiagnosticSeverity.Hint;
        case ts.DiagnosticCategory.Message:
        default:
            return lsp.DiagnosticSeverity.Information;
    }
}
/**
 * Convert PugDocumen errors into LSP diagnostics
 * @param pugDiagnostic
 */
function pugDiagnosticToLspDiagnostic(pugDiagnostic) {
    if (!pugDiagnostic) {
        return;
    }
    let message = `${pugDiagnostic.code}`;
    if (pugDiagnostic.msg) {
        message += `: ${pugDiagnostic.msg.replace(/\n/g, "\\n")}`;
    }
    return lsp.Diagnostic.create(lsp.Range.create(pugDiagnostic.line, pugDiagnostic.column + 1, pugDiagnostic.line, pugDiagnostic.column + 1), message, lsp.DiagnosticSeverity.Error, pugDiagnostic.code);
}
exports.pugDiagnosticToLspDiagnostic = pugDiagnosticToLspDiagnostic;
function _tsDiagnosticToLspDiagnosticTextDocument(tsDiag, document) {
    const textSpan = {
        start: tsDiag.start || 0,
        length: tsDiag.length || 0,
    };
    return lsp.Diagnostic.create(utils.tsTextSpanToLspRange(document, textSpan), ts.flattenDiagnosticMessageText(tsDiag.messageText, '\n'), tsDiagnosticCategoryToLspDiagnosticSeverity(tsDiag.category), tsDiag.code, tsDiag.source, utils.tsRelatedInformationToLspRelatedInformation(document, tsDiag.relatedInformation));
}
function _tsDiagnosticToLspDiagnosticScriptInfo(tsDiag, scriptInfo) {
    const textSpan = {
        start: tsDiag.start || 0,
        length: tsDiag.length || 0,
    };
    return lsp.Diagnostic.create(utils.tsTextSpanToLspRange(scriptInfo, textSpan), ts.flattenDiagnosticMessageText(tsDiag.messageText, '\n'), tsDiagnosticCategoryToLspDiagnosticSeverity(tsDiag.category), tsDiag.code, tsDiag.source, utils.tsRelatedInformationToLspRelatedInformation(scriptInfo, tsDiag.relatedInformation));
}
/**
 * Convert ts.Diagnostic to lsp.Diagnostic
 * @param tsDiag TS diagnostic
 * @param file Used to compute proper offset.
 */
function tsDiagnosticToLspDiagnostic(tsDiag, file) {
    if (file instanceof ts.server.ScriptInfo) {
        return _tsDiagnosticToLspDiagnosticScriptInfo(tsDiag, file);
    }
    else {
        return _tsDiagnosticToLspDiagnosticTextDocument(tsDiag, file);
    }
}
exports.tsDiagnosticToLspDiagnostic = tsDiagnosticToLspDiagnostic;
//# sourceMappingURL=diagnostic.js.map