"use strict";
/**
 * @license
 * Copyright Google Inc. All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process = __importStar(require("child_process"));
const events_1 = require("events");
const path_1 = require("path");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const ngcc_1 = require("../ngcc");
const PACKAGE_ROOT = (0, path_1.resolve)(__dirname, '../../..');
const WORKSPACE_ROOT = (0, path_1.join)(PACKAGE_ROOT, 'integration', 'workspace');
const PROJECT = (0, path_1.join)(WORKSPACE_ROOT, 'projects', 'demo');
const PRE_APF_PROJECT = (0, path_1.join)(PACKAGE_ROOT, 'integration', 'pre_apf_project');
describe('resolveAndRunNgcc', () => {
    const afterEach$ = new rxjs_1.Subject();
    afterEach(() => {
        afterEach$.next();
    });
    it('runs ngcc from node_modules where ngcc is resolved to', async () => {
        const fakeChild = new events_1.EventEmitter();
        const spy = spyOn(child_process, 'fork').and.returnValue(fakeChild);
        // Note that tsconfig.json is in the project directory
        const tsconfig = (0, path_1.join)(PROJECT, 'tsconfig.json');
        // Because resolveNgcc is async, we need to periodically emit `close` from the child since
        // `resolveAndRunNgcc` subscribes after the async resolveNgcc.
        (0, rxjs_1.interval)(500).pipe((0, operators_1.takeUntil)(afterEach$)).subscribe(() => {
            fakeChild.emit('close', 0 /* exit code */);
        });
        const promise = (0, ngcc_1.resolveAndRunNgcc)(tsconfig, { report() { } });
        await expectAsync(promise).toBeResolved();
        expect(spy.calls.count()).toBe(1);
        // First arg is the ngcc binary, second arg is program arguments, third
        // arg is fork options.
        const { cwd } = spy.calls.argsFor(0)[2];
        // cwd for ngcc should be in the workspace directory
        expect(cwd).toBe(WORKSPACE_ROOT);
    });
    it('runs ngcc from node_modules for v12 project where ngcc is resolved to', async () => {
        const fakeChild = new events_1.EventEmitter();
        const spy = spyOn(child_process, 'fork').and.returnValue(fakeChild);
        // Note that tsconfig.json is in the project directory
        const tsconfig = (0, path_1.join)(PRE_APF_PROJECT, 'tsconfig.json');
        // Because resolveNgcc is async, we need to periodically emit `close` from the child since
        // `resolveAndRunNgcc` subscribes after the async resolveNgcc.
        (0, rxjs_1.interval)(500).pipe((0, operators_1.takeUntil)(afterEach$)).subscribe(() => {
            fakeChild.emit('close', 0 /* exit code */);
        });
        const promise = (0, ngcc_1.resolveAndRunNgcc)(tsconfig, { report() { } });
        await expectAsync(promise).toBeResolved();
        expect(spy.calls.count()).toBe(1);
        // First arg is the ngcc binary, second arg is program arguments, third
        // arg is fork options.
        const { cwd } = spy.calls.argsFor(0)[2];
        // cwd for ngcc should be in the workspace directory
        expect(cwd).toBe(PRE_APF_PROJECT);
    });
});
//# sourceMappingURL=ngcc_spec.js.map