"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.translateRange = exports.translateHtmlRangeToPug = exports.translateHtmlPositionToPug = exports.translatePugPositionToHtml = exports.translatePosition = exports.translatePugOffsetToHtml = exports.translateHtmlOffsetToPug = void 0;
const utils_1 = require("./utils");
function translateHtmlOffsetToPug(originOffset, sourceMap) {
    return (0, utils_1.extractOffsetFromSourceMapPosition)(sourceMap.getSourceRange(originOffset)) || originOffset;
}
exports.translateHtmlOffsetToPug = translateHtmlOffsetToPug;
function translatePugOffsetToHtml(originOffset, sourceMap) {
    return (0, utils_1.extractOffsetFromSourceMapPosition)(sourceMap.getMappedRange(originOffset)) || originOffset;
}
exports.translatePugOffsetToHtml = translatePugOffsetToHtml;
function translatePosition(originPosition, originTextDocument, targetTextDocument, sourceMap, sourceMapFunction) {
    const originOffset = originTextDocument.offsetAt({ line: originPosition.line, character: originPosition.character }) + 0;
    const targetOffset = (0, utils_1.extractOffsetFromSourceMapPosition)(sourceMap[sourceMapFunction](originOffset)) || originOffset;
    const targetPosition = targetTextDocument.positionAt(targetOffset);
    return targetPosition;
}
exports.translatePosition = translatePosition;
function translatePugPositionToHtml(originalPosition, pugDocument) {
    return translatePosition(originalPosition, pugDocument.pugTextDocument, pugDocument.htmlTextDocument, pugDocument.sourceMap, 'getMappedRange');
}
exports.translatePugPositionToHtml = translatePugPositionToHtml;
function translateHtmlPositionToPug(originalPosition, pugDocument) {
    return translatePosition(originalPosition, pugDocument.htmlTextDocument, pugDocument.pugTextDocument, pugDocument.sourceMap, 'getSourceRange');
}
exports.translateHtmlPositionToPug = translateHtmlPositionToPug;
function translateHtmlRangeToPug(originalRange, pugDocument) {
    return translateRange(originalRange, pugDocument.htmlTextDocument, pugDocument.pugTextDocument, pugDocument.sourceMap, 'getSourceRange');
}
exports.translateHtmlRangeToPug = translateHtmlRangeToPug;
function translateRange(originRange, originTextDocument, targetTextDocument, sourceMap, sourceMapFunction) {
    const calculateOriginOffset = (position) => originTextDocument.offsetAt({ line: position.line, character: position.character });
    const originStartOffset = calculateOriginOffset(originRange.start);
    const originEndOffset = calculateOriginOffset(originRange.end);
    const sourceMapResult = sourceMap[sourceMapFunction](originStartOffset, originEndOffset);
    let targetEndOffset = 0;
    let targetStartOffset = 0;
    if (sourceMapResult && (sourceMapResult === null || sourceMapResult === void 0 ? void 0 : sourceMapResult[0])) {
        targetStartOffset = sourceMapResult[0].start;
        targetEndOffset = sourceMapResult[0].end;
    }
    const targetRange = {
        start: targetTextDocument.positionAt(targetStartOffset),
        end: targetTextDocument.positionAt(targetEndOffset),
    };
    return targetRange;
}
exports.translateRange = translateRange;
//# sourceMappingURL=translation.js.map