"use strict";
/**
 * @license
 * Copyright Google Inc. All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tsRelatedInformationToLspRelatedInformation = exports.lspRangeToTsPositions = exports.tsTextSpanToLspRange = exports.lspPositionToTsPosition = exports.getMappedDefinitionInfo = exports.tsDisplayPartsToText = exports.MruTracker = exports.isConfiguredProject = exports.getPugDocumentFromScriptInfo = exports.extractOffsetFromSourceMapPosition = exports.filePathToUri = exports.uriToFilePath = exports.isDebugMode = exports.translateRange = exports.translatePugPositionToHtml = exports.translatePugOffsetToHtml = exports.translatePosition = exports.translateHtmlRangeToPug = exports.translateHtmlPositionToPug = exports.translateHtmlOffsetToPug = void 0;
const ts = __importStar(require("typescript/lib/tsserverlibrary"));
const vscode_uri_1 = require("vscode-uri");
const textDocument_1 = require("./textDocument");
const scriptInfo_1 = require("./scriptInfo");
var translation_1 = require("./translation");
Object.defineProperty(exports, "translateHtmlOffsetToPug", { enumerable: true, get: function () { return translation_1.translateHtmlOffsetToPug; } });
Object.defineProperty(exports, "translateHtmlPositionToPug", { enumerable: true, get: function () { return translation_1.translateHtmlPositionToPug; } });
Object.defineProperty(exports, "translateHtmlRangeToPug", { enumerable: true, get: function () { return translation_1.translateHtmlRangeToPug; } });
Object.defineProperty(exports, "translatePosition", { enumerable: true, get: function () { return translation_1.translatePosition; } });
Object.defineProperty(exports, "translatePugOffsetToHtml", { enumerable: true, get: function () { return translation_1.translatePugOffsetToHtml; } });
Object.defineProperty(exports, "translatePugPositionToHtml", { enumerable: true, get: function () { return translation_1.translatePugPositionToHtml; } });
Object.defineProperty(exports, "translateRange", { enumerable: true, get: function () { return translation_1.translateRange; } });
exports.isDebugMode = process.env['NG_DEBUG'] === 'true';
var Scheme;
(function (Scheme) {
    Scheme["File"] = "file";
})(Scheme || (Scheme = {}));
/**
 * Extract the file path from the specified `uri`.
 * @param uri
 */
function uriToFilePath(uri) {
    // Note: uri.path is different from uri.fsPath
    // See
    // https://github.com/microsoft/vscode-uri/blob/413805221cc6ed167186ab3103d3248d6f7161f2/src/index.ts#L622-L645
    const { scheme, fsPath } = vscode_uri_1.URI.parse(uri);
    if (scheme !== Scheme.File) {
        return '';
    }
    return fsPath;
}
exports.uriToFilePath = uriToFilePath;
/**
 * Converts the specified `filePath` to a proper URI.
 * @param filePath
 */
function filePathToUri(filePath) {
    return vscode_uri_1.URI.file(filePath).toString();
}
exports.filePathToUri = filePathToUri;
/**
 * Get the starting offset/position from a source map result
 * @param sourceMapPosition
 */
function extractOffsetFromSourceMapPosition(sourceMapPosition) {
    return sourceMapPosition === null || sourceMapPosition === void 0 ? void 0 : sourceMapPosition[0].start;
}
exports.extractOffsetFromSourceMapPosition = extractOffsetFromSourceMapPosition;
/**
 * Get the parsed PugDocument that corresponds to a ts.server.ScriptInfo
 * @param pugLs
 * @param scriptInfo
 * @param scriptInfo
 */
function getPugDocumentFromScriptInfo(pugLs, scriptInfo) {
    const documentSnapshot = scriptInfo.getSnapshot();
    const documentText = documentSnapshot
        .getText(0, documentSnapshot.getLength());
    return pugLs.parsePugDocument(documentText);
}
exports.getPugDocumentFromScriptInfo = getPugDocumentFromScriptInfo;
/**
 * Check if a project is configured
 * @param project
 */
function isConfiguredProject(project) {
    return project.projectKind === ts.server.ProjectKind.Configured;
}
exports.isConfiguredProject = isConfiguredProject;
/**
 * A class that tracks items in most recently used order.
 */
class MruTracker {
    constructor() {
        this.set = new Set();
    }
    update(item) {
        if (this.set.has(item)) {
            this.set.delete(item);
        }
        this.set.add(item);
    }
    delete(item) {
        this.set.delete(item);
    }
    /**
     * Returns all items sorted by most recently used.
     */
    getAll() {
        // Javascript Set maintains insertion order, see
        // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Set
        // Since items are sorted from least recently used to most recently used,
        // we reverse the result.
        return [...this.set].reverse();
    }
}
exports.MruTracker = MruTracker;
/**
 * Combine ts.SymboleDisplayParts into a regular string
 * @param parts
 */
function tsDisplayPartsToText(parts) {
    return parts.map(dp => dp.text).join('');
}
exports.tsDisplayPartsToText = tsDisplayPartsToText;
/**
 *
 * This function attempts to use *internal* TypeScript APIs to find the original source spans for
 * the `ts.DefinitionInfo` using source maps. If it fails, this function returns the same
 * `ts.DefinitionInfo` that was passed in.
 *
 * @see https://github.com/angular/vscode-ng-language-service/issues/1588
 */
function getMappedDefinitionInfo(info, project) {
    try {
        const mappedDocumentSpan = getMappedDocumentSpan(info, project);
        return Object.assign(Object.assign({}, info), mappedDocumentSpan);
    }
    catch (_a) {
        return info;
    }
}
exports.getMappedDefinitionInfo = getMappedDefinitionInfo;
function getMappedDocumentSpan(documentSpan, project) {
    const newPosition = getMappedLocation(documentSpanLocation(documentSpan), project);
    if (!newPosition)
        return undefined;
    return {
        fileName: newPosition.fileName,
        textSpan: { start: newPosition.pos, length: documentSpan.textSpan.length },
        originalFileName: documentSpan.fileName,
        originalTextSpan: documentSpan.textSpan,
        contextSpan: getMappedContextSpan(documentSpan, project),
        originalContextSpan: documentSpan.contextSpan
    };
}
function getMappedLocation(location, project) {
    const mapsTo = project.getSourceMapper().tryGetSourcePosition(location);
    return mapsTo &&
        project.projectService.fileExists(ts.server.toNormalizedPath(mapsTo.fileName)) ?
        mapsTo :
        undefined;
}
function documentSpanLocation({ fileName, textSpan }) {
    return { fileName, pos: textSpan.start };
}
function getMappedContextSpan(documentSpan, project) {
    const contextSpanStart = documentSpan.contextSpan &&
        getMappedLocation({ fileName: documentSpan.fileName, pos: documentSpan.contextSpan.start }, project);
    const contextSpanEnd = documentSpan.contextSpan &&
        getMappedLocation({ fileName: documentSpan.fileName, pos: documentSpan.contextSpan.start + documentSpan.contextSpan.length }, project);
    return contextSpanStart && contextSpanEnd ?
        { start: contextSpanStart.pos, length: contextSpanEnd.pos - contextSpanStart.pos } :
        undefined;
}
/**
 * Convert lsp.Position to the absolute offset in the file. LSP keeps track of
 * offset using 0-based index whereas TypeScript uses 1-based index.
 * @param file Used to determine the offsets.
 * @param position
 */
function lspPositionToTsPosition(file, position) {
    if (file instanceof ts.server.ScriptInfo) {
        return (0, scriptInfo_1._lspPositionToTsPositionScriptInfo)(file, position);
    }
    else {
        return (0, textDocument_1._lspPositionToTsPositionTextDocument)(file, position);
    }
}
exports.lspPositionToTsPosition = lspPositionToTsPosition;
/**
 * Convert ts.TextSpan to lsp.TextSpan. TypeScript keeps track of offset using
 * 1-based index whereas LSP uses 0-based index.
 * @param file Used to determine the offsets.
 * @param textSpan
 */
function tsTextSpanToLspRange(file, textSpan) {
    if (file instanceof ts.server.ScriptInfo) {
        return (0, scriptInfo_1._tsTextSpanToLspRangeScriptInfo)(file, textSpan);
    }
    else {
        return (0, textDocument_1._tsTextSpanToLspRangeTextDocument)(file, textSpan);
    }
}
exports.tsTextSpanToLspRange = tsTextSpanToLspRange;
/**
 * Convert lsp.Range which is made up of `start` and `end` positions to
 * TypeScript's absolute offsets.
 * @param file Used to determine the offsets.
 * @param range
 */
function lspRangeToTsPositions(file, range) {
    return (0, textDocument_1._lspRangeToTsPositionsTextDocument)(file, range);
}
exports.lspRangeToTsPositions = lspRangeToTsPositions;
/**
 * Convert a ts.DiagnosticRelatedInformation array to a
 * lsp.DiagnosticRelatedInformation array
 * @param file Used to determine the offsets.
 * @param relatedInfo
 */
function tsRelatedInformationToLspRelatedInformation(file, relatedInfo) {
    if (file instanceof ts.server.ScriptInfo) {
        return (0, scriptInfo_1._tsRelatedInformationToLspRelatedInformationScriptInfo)(file, relatedInfo);
    }
    else {
        return (0, textDocument_1._tsRelatedInformationToLspRelatedInformationTextDocument)(file, relatedInfo);
    }
}
exports.tsRelatedInformationToLspRelatedInformation = tsRelatedInformationToLspRelatedInformation;
//# sourceMappingURL=utils.js.map