"use strict";
/**
 * @license
 * Copyright Google Inc. All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveNgcc = exports.resolveNgLangSvc = exports.loadEsmModule = exports.resolveTsServer = void 0;
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const url_1 = __importDefault(require("url"));
const resolver_1 = require("../common/resolver");
const MIN_TS_VERSION = '4.3';
const MIN_NG_VERSION = '13.0';
const TSSERVERLIB = 'typescript/lib/tsserverlibrary';
/**
 * Resolve the node module with the specified `packageName` that satisfies
 * the specified minimum version.
 * @param packageName name of package to be resolved
 * @param minVersionStr minimum version
 * @param probeLocations locations to initiate node module resolution
 * @param rootPackage location of package.json. For example, the root package of
 * `typescript/lib/tsserverlibrary` is `typescript`.
 */
function resolveWithMinVersion(packageName, minVersionStr, probeLocations, rootPackage) {
    if (!packageName.startsWith(rootPackage)) {
        throw new Error(`${packageName} must be in the root package`);
    }
    const minVersion = new resolver_1.Version(minVersionStr);
    for (const location of probeLocations) {
        const nodeModule = (0, resolver_1.resolve)(packageName, location, rootPackage);
        if (nodeModule && nodeModule.version.greaterThanOrEqual(minVersion)) {
            return nodeModule;
        }
    }
    throw new Error(`Failed to resolve '${packageName}' with minimum version '${minVersion}' from ` +
        JSON.stringify(probeLocations, null, 2));
}
/**
 * Resolve `typescript/lib/tsserverlibrary` from the given locations.
 * @param probeLocations
 */
function resolveTsServer(probeLocations) {
    if (probeLocations.length > 0) {
        // The first probe location is `typescript.tsdk` if it is specified.
        const resolvedFromTsdk = resolveTsServerFromTsdk(probeLocations[0]);
        if (resolvedFromTsdk !== undefined) {
            return resolvedFromTsdk;
        }
    }
    return resolveWithMinVersion(TSSERVERLIB, MIN_TS_VERSION, probeLocations, 'typescript');
}
exports.resolveTsServer = resolveTsServer;
function resolveTsServerFromTsdk(tsdk) {
    // `tsdk` is the folder path to the tsserver and lib*.d.ts files under a
    // TypeScript install, for example
    // - /google/src/head/depot/google3/third_party/javascript/node_modules/typescript/stable/lib
    if (!path.isAbsolute(tsdk)) {
        return undefined;
    }
    const tsserverlib = path.join(tsdk, 'tsserverlibrary.js');
    if (!fs.existsSync(tsserverlib)) {
        return undefined;
    }
    const packageJson = path.resolve(tsserverlib, '../../package.json');
    if (!fs.existsSync(packageJson)) {
        return undefined;
    }
    try {
        const json = JSON.parse(fs.readFileSync(packageJson, 'utf8'));
        return {
            name: TSSERVERLIB,
            resolvedPath: tsserverlib,
            version: new resolver_1.Version(json.version),
        };
    }
    catch (_a) {
        return undefined;
    }
}
/**
 * This uses a dynamic import to load a module which may be ESM.
 * CommonJS code can load ESM code via a dynamic import. Unfortunately, TypeScript
 * will currently, unconditionally downlevel dynamic import into a require call.
 * require calls cannot load ESM code and will result in a runtime error. To workaround
 * this, a Function constructor is used to prevent TypeScript from changing the dynamic import.
 * Once TypeScript provides support for keeping the dynamic import this workaround can
 * be dropped.
 *
 * @param modulePath The path of the module to load.
 * @returns A Promise that resolves to the dynamically imported module.
 */
function loadEsmModule(modulePath) {
    return new Function('modulePath', `return import(modulePath);`)(modulePath);
}
exports.loadEsmModule = loadEsmModule;
/**
 * Resolve `pugjs-angular-language-service` from the given locations.
 * @param probeLocations locations from which resolution is attempted
 */
function resolveNgLangSvc(probeLocations) {
    const ngls = 'pugjs-angular-language-service';
    return resolveWithMinVersion(ngls, MIN_NG_VERSION, probeLocations, ngls);
}
exports.resolveNgLangSvc = resolveNgLangSvc;
async function resolveNgcc(directory) {
    // Try to resolve ngcc from the new package format since the v13 release
    try {
        const ngcc = (0, resolver_1.resolve)('@angular/compiler-cli/ngcc', directory, '@angular/compiler-cli');
        if (ngcc === undefined) {
            throw new Error('Could not resolve ngcc');
        }
        // The Angular compiler-CLI package is strict ESM as of v13.
        const ngccModule = await loadEsmModule(url_1.default.pathToFileURL(ngcc.resolvedPath));
        const resolvedPath = ngccModule.ngccMainFilePath;
        if (resolvedPath === undefined) {
            throw new Error('could not resolve ngcc path.');
        }
        return Object.assign(Object.assign({}, ngcc), { resolvedPath });
    }
    catch (e) {
        // Try to resolve ngcc through with pre-v13 package format
        return (0, resolver_1.resolve)('@angular/compiler-cli/ngcc/main-ngcc.js', directory, '@angular/compiler-cli');
    }
}
exports.resolveNgcc = resolveNgcc;
//# sourceMappingURL=version_provider.js.map